﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Microsoft.Win32;

namespace RGBConfigurator
{
    public partial class Form1 : Form
    {
        private usb_api.usb_interface usbInterface = new usb_api.usb_interface();

        private bool connected = false;

        private bool stationLocked = false;

        public Form1()
        {
            InitializeComponent();

            timer1.Start();
            fixedControl2.ModeNo = 0;
            fixedControl2.ModeName = "Fast RGB-Change";
            fixedControl3.ModeNo = 1;
            fixedControl3.ModeName = "Slow RGB-Change";
            temperatureColorControl1.ModeNo = 2;
            constantColorControl1.ModeNo = 3;
            constantColorControl2.ModeNo = 4;
            constantColorControl3.ModeNo = 5;
            constantColorControl4.ModeNo = 6;


            SystemEvents.SessionSwitch +=
	          new SessionSwitchEventHandler(OnSessionSwitch);

        }

        private void beendenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void buttonWrite_Click(object sender, EventArgs e)
        {
            byte[] data = new byte[14];

            data[0] = temperatureColorControl1.TempMin;
            data[1] = temperatureColorControl1.TempMax;

            GetColorData(2, data, constantColorControl1);
            GetColorData(5, data, constantColorControl2);
            GetColorData(8, data, constantColorControl3);
            GetColorData(11, data, constantColorControl4);


            //Write Data to PIC
            byte[] result;
            usbInterface.EasyCommand(0x01, 1, data, out result);
            if (result == null)
            {
                MessageBox.Show("Could not write configuration.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void GetColorData(int p, byte[] data, ConstantColorControl constantColorControl1)
        {
            data[p] = constantColorControl1.R;
            data[p+1] = constantColorControl1.G;
            data[p+2] = constantColorControl1.B;
        }

        private void SetColorData(int p, byte[] data, ConstantColorControl constantColorControl1)
        {
            constantColorControl1.R = data[p];
            constantColorControl1.G = data[p + 1];
            constantColorControl1.B = data[p + 2];
        }

        private void buttonRead_Click(object sender, EventArgs e)
        {
            //Read Data from PIC
            byte[] result;
            usbInterface.EasyCommand(0x02, 16, new byte[1], out result);
            if (result != null)
            {
                temperatureColorControl1.TempMin = result[1];
                temperatureColorControl1.TempMax = result[2];

                SetColorData(3, result, constantColorControl1);
                SetColorData(6, result, constantColorControl2);
                SetColorData(9, result, constantColorControl3);
                SetColorData(12, result, constantColorControl4);

            }
            else
            {
                MessageBox.Show("Could not read configuration.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        /**
         *  Check connection state by qerying the version.
         *  The CPU load based color is transmitted also, if selected.
         */
        private void timer1_Tick(object sender, EventArgs e)
        {

            
            byte[] result;
            byte[] data = new byte[5];

            if ((cpuColorControl1.cpuModeActive || cpuColorControl1.ambilightModeActive) && stationLocked)
            {
                data[0] = 1;
                data[1] = 0;
                data[2] = 0;
                data[3] = 255;
                data[4] = 0; //Change color immediately

            }
            else if (cpuColorControl1.cpuModeActive)
            {
                Color c = cpuColorControl1.GetCPULoadColor();
                data[0] = 1;
                data[1] = c.R;
                data[2] = c.G;
                data[3] = c.B;
                data[4] = 3; //Change color with delay level 3 (slower than 0, faster than 1)
            }
            else if(cpuColorControl1.ambilightModeActive)
            {
                Color c;
                try
                {
                    c = cpuColorControl1.GetAmbilightColor();
                }
                catch (Exception)
                {
                    c = Color.Red;
                }
                data[0] = 1;
                data[1] = c.R;
                data[2] = c.G;
                data[3] = c.B;
                data[4] = 3; //Change color with delay level 3 (slower than 0, faster than 1)
            }
            else
            {
                data[0] = 0;
            }


            //Test connection to PIC
            usbInterface.EasyCommand(0x00, 4, data, out result);
            if (result == null)
            {
                toolStripModeLabel.Text = "Mode: ?";
                toolStripStatusLabel1.Text = "Status: not connected";
                toolStripStatusLabel1.BackColor = Color.Red;
                connected = false;
                toolStripStatusLabelTemperature.Text = "Temperature: ?°C";
            }
            else
            {
                int mode = result[1];
                toolStripModeLabel.Text = "Mode: " + mode;
                toolStripStatusLabel1.Text = "Status: connected";
                toolStripStatusLabel1.BackColor = Color.Green;
                if (mode == 255)
                {
                    //If this is the initial start of the pic program it with the default values....
                    buttonWrite.PerformClick();
                    return;
                }
                if(!connected)
                {
                    //Read configuration if connection is established first time
                    buttonRead.PerformClick();
                }
                connected = true;                
                double temp = ((double)result[2])/2;
                toolStripStatusLabelTemperature.Text = "Temperature: " + temp + "°C";

            }
        }


        void OnSessionSwitch(object sender, Microsoft.Win32.SessionSwitchEventArgs e)
        {
            if (e.Reason == SessionSwitchReason.SessionLock)
            {
                stationLocked = true;
            }
            else if (e.Reason == SessionSwitchReason.SessionUnlock)
            {
                stationLocked = false;
            }
        }
      
    }

  

   

}